package com.arms.api.issue.almapi.service;

import com.arms.api.issue.almapi.model.dto.지라사용자_데이터;
import com.arms.api.issue.almapi.model.dto.AlmIssueCreationDTO;
import com.arms.api.issue.almapi.model.dto.IssueCreationFieldsDTO;
import com.arms.api.issue.almapi.model.dto.지라이슈필드_데이터;
import com.arms.api.issue.almapi.model.entity.AlmIssueEntity;
import com.arms.api.issue.almapi.model.vo.지라이슈_데이터;
import com.arms.api.issue.almapi.repository.지라이슈_저장소;
import com.arms.api.issue.priority.model.IssuePriorityDTO;
import com.arms.api.issue.status.dto.IssueStatusDTO;
import com.arms.api.issue.type.dto.IssueTypeDTO;
import com.arms.api.project.dto.ProjectDTO;
import com.arms.api.serverinfo.model.서버정보_데이터;
import com.arms.api.util.alm.레드마인API_정보;
import com.arms.api.util.alm.레드마인유틸;
import com.arms.egovframework.javaservice.esframework.erros.response.ErrorCode;
import com.arms.api.util.errors.ErrorLogUtil;
import com.taskadapter.redmineapi.*;
import com.taskadapter.redmineapi.bean.Issue;
import com.taskadapter.redmineapi.bean.IssueRelation;
import com.taskadapter.redmineapi.bean.Tracker;
import com.taskadapter.redmineapi.bean.User;

import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Component;

import java.time.LocalDate;
import java.time.ZonedDateTime;
import java.time.format.DateTimeFormatter;
import java.util.*;
import java.util.stream.Collectors;

import static com.arms.api.util.alm.지라유틸.isDateInRange;
import static com.arms.api.util.alm.지라유틸.toLocalDate;
import static java.util.stream.Collectors.toList;

@Slf4j
@Component
@AllArgsConstructor
public class 온프레미스_레드마인_이슈전략 implements 이슈전략 {

    private final 레드마인유틸 레드마인유틸;
    private final 레드마인API_정보 레드마인API_정보;
    private final 지라이슈_저장소 지라이슈_저장소;


    @Override
    public List<지라이슈_데이터> getIssueList(서버정보_데이터 서버정보, String projectKeyOrId) {

        RedmineManager redmineManager = 레드마인유틸.레드마인_온프레미스_통신기_생성(서버정보.getUri(), 서버정보.getPasswordOrToken());

        List<Issue> issueList;
        try {
            issueList = redmineManager.getIssueManager().getIssues(projectKeyOrId, null);
        }
        catch (RedmineException e) {
            String errorMessage = ErrorLogUtil.exceptionLoggingAndReturn(e, this.getClass().getName(),
                String.format(" %s[%s] :: 프로젝트[%s] :: getIssueList 중 오류", 서버정보.getType(), 서버정보.getUri(), projectKeyOrId));
            throw new IllegalArgumentException(errorMessage);
        }

        List<지라이슈_데이터> returnIssueList = issueList.stream().map(이슈 -> {
                지라이슈_데이터 지라이슈_데이터 = convertRedmineIssueToAlmIssue(redmineManager, 이슈, 서버정보.getUri());
                return 지라이슈_데이터;
            })
            .filter(Objects::nonNull)
            .collect(toList());

        return returnIssueList;
    }

    @Override
    public 지라이슈_데이터 createIssue(서버정보_데이터 서버정보, AlmIssueCreationDTO almIssueCreationDTO) {

        RedmineManager redmineManager = 레드마인유틸.레드마인_온프레미스_통신기_생성(서버정보.getUri(), 서버정보.getPasswordOrToken());

        IssueCreationFieldsDTO fieldsDTO = almIssueCreationDTO.getFields();
        if (fieldsDTO == null) {
            String errorMessage = String.format("%s[%s] :: 이슈 생성 필드 데이터가 존재 하지 않습니다.",
                서버정보.getType(), 서버정보.getUri());
            log.error(errorMessage);
            throw new IllegalArgumentException(ErrorCode.요청본문_오류체크.getErrorMsg() + " :: " + errorMessage);
        }

        String projectKeyOrId = null;
        String issueTypeId = null;
        String issueTypeName = null;

        if (fieldsDTO.getProject()!= null && StringUtils.isNotEmpty(fieldsDTO.getProject().getId())) {
            projectKeyOrId = fieldsDTO.getProject().getId();
        }

        if (fieldsDTO.getIssuetype() != null
            && StringUtils.isNotEmpty(fieldsDTO.getIssuetype().getId())
            && StringUtils.isNotEmpty(fieldsDTO.getIssuetype().getName())) {
            issueTypeId = fieldsDTO.getIssuetype().getId();
            issueTypeName = fieldsDTO.getIssuetype().getName();
        }

        if (StringUtils.isEmpty(projectKeyOrId) || issueTypeId == null || issueTypeName == null) {
            String errorMessage = String.format("%s :: %s 이슈 생성 필드 확인에 필요한 프로젝트 아이디, 이슈유형 아이디가 존재하지 않습니다.",
                서버정보.getType(), 서버정보.getUri());
            throw new IllegalArgumentException(errorMessage);
        }

        Tracker issueType = new Tracker()
            .setId(Integer.parseInt(issueTypeId))
            .setName(issueTypeName);

        Issue issue = new Issue(redmineManager.getTransport(), Integer.parseInt(projectKeyOrId))
            .setTracker(issueType);

        if (StringUtils.isNotEmpty(fieldsDTO.getSummary())) {
            issue.setSubject(fieldsDTO.getSummary());
        }

        if (StringUtils.isNotEmpty(fieldsDTO.getDescription())) {
            issue.setDescription(fieldsDTO.getDescription());
        }

        if (fieldsDTO.getPriority() != null && StringUtils.isNotEmpty(fieldsDTO.getPriority().getId())) {
            issue.setPriorityId(Integer.parseInt(fieldsDTO.getPriority().getId()));
        }

        if (fieldsDTO.getStartDate() != null) {
            issue.setStartDate(fieldsDTO.getStartDate());
        }

        if (fieldsDTO.getDueDate() != null) {
            issue.setDueDate(fieldsDTO.getDueDate());
        }

        try {
            issue = issue.create();
        }
        catch (RedmineException e) {
            String errorMessage = ErrorLogUtil.exceptionLoggingAndReturn(e, this.getClass().getName(),
                String.format("%s[%s],  프로젝트[%s], 이슈유형[%s], 생성 필드 :: 이슈 생성하기 중 오류",
                    서버정보.getType(), 서버정보.getUri(), projectKeyOrId, issueTypeId, issue.toString()));
            throw new IllegalArgumentException(ErrorCode.이슈생성_오류.getErrorMsg() + " :: " +errorMessage);
        }

        지라이슈_데이터 almIssueData = convertRedmineIssueToAlmIssue(redmineManager, issue, 서버정보.getUri());
        return almIssueData;
    }

    @Override
    public Map<String, Object> 이슈_수정하기(서버정보_데이터 서버정보, String 이슈_키_또는_아이디, AlmIssueCreationDTO almIssueCreationDTO) {

        RedmineManager 레드마인_매니저 = 레드마인유틸.레드마인_온프레미스_통신기_생성(서버정보.getUri(), 서버정보.getPasswordOrToken());

        Map<String, Object> 결과 = new HashMap<>();
        IssueCreationFieldsDTO 필드_데이터 = almIssueCreationDTO.getFields();
        String 제목 = 필드_데이터.getSummary();
        String 내용 = 필드_데이터.getDescription();
        IssueStatusDTO 상태_데이터 = 필드_데이터.getStatus();

        try {
            Issue 수정이슈 = 레드마인_매니저.getIssueManager().getIssueById(Integer.parseInt(이슈_키_또는_아이디));

            if (제목 != null && !제목.isEmpty()) {
                수정이슈.setSubject(제목);
            }
            if (내용 != null) {
                수정이슈.setDescription(내용);
            }

            수정이슈.update();

            결과.put("success", true);
            결과.put("message", "이슈 수정 성공");
        }
        catch (RedmineException e) {
            String 에러로그 = ErrorLogUtil.exceptionLoggingAndReturn(e, this.getClass().getName(),
                "레드마인_온프레미스 ["+ 서버정보.getUri() +"] :: 이슈_키_또는_아이디 :: "
                    + 이슈_키_또는_아이디+ " :: 수정데이터 :: " + 필드_데이터.toString() + "이슈_수정하기 오류");
            log.error(에러로그);

            결과.put("success", false);
            결과.put("message", 에러로그);
        }

        return 결과;
    }

    @Override
    public Map<String, Object> 이슈_상태_변경하기(서버정보_데이터 서버정보, String 이슈_키_또는_아이디, String 상태_아이디) {

        RedmineManager 레드마인_매니저 = 레드마인유틸.레드마인_온프레미스_통신기_생성(서버정보.getUri(), 서버정보.getPasswordOrToken());

        Map<String, Object> 결과 = new HashMap<>();

        try {
            Issue 수정이슈 = 레드마인_매니저.getIssueManager().getIssueById(Integer.parseInt(이슈_키_또는_아이디));

            수정이슈.setStatusId(Integer.parseInt(상태_아이디));
            수정이슈.update();

            boolean 상태_변경_확인 = 이슈_상태_검증하기(서버정보, 이슈_키_또는_아이디, 상태_아이디);

            if (상태_변경_확인) {
                결과.put("success", true);
                결과.put("message", "이슈 상태 변경 성공");
            } else {
                String 에러로그 = "레드마인_온프레미스 [" + 서버정보.getUri() + "] :: 이슈 키[" + 이슈_키_또는_아이디 + "] :: 상태 아이디[" + 상태_아이디 + "] :: 해당 업무 흐름으로 변경이 불가능 합니다.";
                log.error(에러로그);

                결과.put("success", false);
                결과.put("message", "변경할 이슈 상태가 존재하지 않습니다.");
            }

        }
        catch (RedmineException e) {
            String 에러로그 = ErrorLogUtil.exceptionLoggingAndReturn(e, this.getClass().getName(),
                "레드마인_온프레미스 [" + 서버정보.getUri() + "] :: 이슈_키_또는_아이디 :: "
                    + 이슈_키_또는_아이디 + " :: 수정데이터 :: " + 상태_아이디 + "이슈_상태_변경하기 오류");
            log.error(에러로그);

            결과.put("success", false);
            결과.put("message", 에러로그);
        }

        return 결과;
    }

    public boolean 이슈_상태_검증하기(서버정보_데이터 서버정보, String 이슈_키_또는_아이디, String 상태_아이디) {

        RedmineManager 레드마인_매니저 = 레드마인유틸.레드마인_온프레미스_통신기_생성(서버정보.getUri(), 서버정보.getPasswordOrToken());

        try {
            Issue 수정한_이슈 = 레드마인_매니저.getIssueManager().getIssueById(Integer.parseInt(이슈_키_또는_아이디));

            return Optional.ofNullable(수정한_이슈)
                .map(Issue::getStatusId)
                .map(현재상태 -> StringUtils.equals(상태_아이디, String.valueOf(현재상태)))
                .orElse(false);
        }
        catch (RedmineException e) {
            ErrorLogUtil.exceptionLoggingAndReturn(e, this.getClass().getName(),
                "레드마인_온프레미스 ["+ 서버정보.getUri() +"] :: 이슈_키_또는_아이디 :: "
                    + 이슈_키_또는_아이디 + " :: 이슈_상태_검증하기 오류");
            return false;
        }
    }

    @Override
    public Map<String, Object> 이슈_삭제하기(서버정보_데이터 서버정보, String 이슈_키_또는_아이디) {

        Map<String, Object> 결과 = new HashMap<>();
        RedmineManager 레드마인_매니저 = 레드마인유틸.레드마인_온프레미스_통신기_생성(서버정보.getUri(), 서버정보.getPasswordOrToken());

        try {
            레드마인_매니저.getIssueManager().deleteIssue(Integer.valueOf(이슈_키_또는_아이디));

            결과.put("success", true);
            결과.put("message", "이슈 삭제 성공");
        }
        catch (RedmineException e) {
            String 에러로그 = ErrorLogUtil.exceptionLoggingAndReturn(e, this.getClass().getName(),
                "레드마인_온프레미스 ["+ 서버정보.getUri() +"] :: 이슈_키_또는_아이디 :: "
                    + 이슈_키_또는_아이디+ " :: 이슈_삭제하기 오류");
            log.error(에러로그);

            결과.put("success", false);
            결과.put("message", 에러로그);
        }

        return 결과;
    }

    @Override
    public 지라이슈_데이터 이슈_상세정보_가져오기(서버정보_데이터 서버정보, String issueKeyOrId) throws Exception {

        RedmineManager redmineManager = 레드마인유틸.레드마인_온프레미스_통신기_생성(서버정보.getUri(), 서버정보.getPasswordOrToken());

        try {
            Issue issue = redmineManager.getIssueManager().getIssueById(Integer.parseInt(issueKeyOrId));

            지라이슈_데이터 almIssueData = convertRedmineIssueToAlmIssue(redmineManager, issue, 서버정보.getUri());
            return almIssueData;
        }
        catch (NotFoundException e) {
            ErrorLogUtil.exceptionLogging(e, this.getClass().getName(),
                "레드마인_온프레미스 ["+ 서버정보.getUri() +"] :: 이슈_키_또는_아이디 :: "
                    + issueKeyOrId+ " :: 이슈_상세정보_가져오기 오류");

            if (e.getMessage() != null && e.getMessage().contains("404")) {
                return null;
            }

            throw e;
        }
        catch (RedmineException e) {
            ErrorLogUtil.exceptionLogging(e, this.getClass().getName(),
                "레드마인_온프레미스 ["+ 서버정보.getUri() +"] :: 이슈_키_또는_아이디 :: "
                    + issueKeyOrId + " :: 이슈_상세정보_가져오기 오류");

            throw e;
        }
    }

    @Override
    public List<지라이슈_데이터> 연결이슈_가져오기(서버정보_데이터 서버정보, String 이슈_키_또는_아이디) throws Exception {

        RedmineManager 레드마인_매니저 = 레드마인유틸.레드마인_온프레미스_통신기_생성(서버정보.getUri(), 서버정보.getPasswordOrToken());

        Set<String> 이슈아이디_세트 = new HashSet<>();
        이슈아이디_세트.add(이슈_키_또는_아이디);

        List<지라이슈_데이터> 연결이슈_목록 = new ArrayList<>();

        연결이슈_가져오기_재귀(레드마인_매니저, 서버정보, 이슈_키_또는_아이디, 이슈아이디_세트, 연결이슈_목록);

        return 연결이슈_목록;
    }

    private void 연결이슈_가져오기_재귀(RedmineManager 레드마인_매니저, 서버정보_데이터 서버정보, String 이슈_키_또는_아이디, Set<String> 이슈아이디_세트, List<지라이슈_데이터> 연결이슈_목록) throws Exception {

        Issue 이슈 = null;
        try {
            이슈 = 레드마인_매니저.getIssueManager().getIssueById(Integer.parseInt(이슈_키_또는_아이디), Include.relations);
        }
        catch (RedmineException e) {
            ErrorLogUtil.exceptionLoggingAndReturn(e, this.getClass().getName(),
                "레드마인_온프레미스 ["+ 서버정보.getUri() +"] :: 이슈_키_또는_아이디 :: "
                    + 이슈_키_또는_아이디+ " :: 연결이슈_가져오기 오류");
            return;
        }

        if (이슈 == null || 이슈.getRelations() == null || 이슈.getRelations().isEmpty()) {
            log.info("레드마인_온프레미스 ["+ 서버정보.getUri() +"] :: 이슈_키_또는_아이디 :: "
                + 이슈_키_또는_아이디 + " :: 연결이슈 데이터가 없습니다.");
            return;
        }

        for (IssueRelation 연결이슈 : 이슈.getRelations()) {

            String 연결이슈_아이디 = null;
            if (이슈_키_또는_아이디.equals(String.valueOf(연결이슈.getIssueId()))) {
                연결이슈_아이디 = String.valueOf(연결이슈.getIssueToId());
            } else {
                연결이슈_아이디 = String.valueOf(연결이슈.getIssueId());
            }

            if (연결이슈_아이디 != null && !이슈아이디_세트.contains(연결이슈_아이디)) {
                이슈아이디_세트.add(연결이슈_아이디);
                지라이슈_데이터 연결이슈_데이터 = 이슈_상세정보_가져오기(서버정보, 연결이슈_아이디);

                // 요구사항 이슈인 경우는 미수집
                연결이슈_데이터.set서버아이디(서버정보.getConnectId());
                String 조회_아이디 = 연결이슈_데이터.조회조건_아이디();

                AlmIssueEntity 조회_결과 = 지라이슈_저장소.findRecentDocById(조회_아이디);

                if (조회_결과 != null && 조회_결과.getIsReq() == Boolean.TRUE) {
                    log.info("레드마인_온프레미스 ["+ 서버정보.getUri() +"] :: 연결이슈_아이디 :: "
                        + 연결이슈_아이디 + " :: 요구사항 이슈입니다.");
                    continue;
                }

                if (연결이슈_데이터 != null) {
                    연결이슈_목록.add(연결이슈_데이터);
                    연결이슈_가져오기_재귀(레드마인_매니저, 서버정보, 연결이슈_아이디, 이슈아이디_세트, 연결이슈_목록);
                }
            }
        }
    }

    @Override
    public List<지라이슈_데이터> 서브테스크_가져오기(서버정보_데이터 서버정보, String 이슈_키_또는_아이디) {

        RedmineManager 레드마인_매니저 = 레드마인유틸.레드마인_온프레미스_통신기_생성(서버정보.getUri(), 서버정보.getPasswordOrToken());
        List<지라이슈_데이터> 하위이슈_목록 = new ArrayList<>();
        서브테스크_가져오기_재귀(레드마인_매니저, 서버정보.getUri(), 이슈_키_또는_아이디, 하위이슈_목록);

        return 하위이슈_목록;
    }

    private void 서브테스크_가져오기_재귀(RedmineManager 레드마인_매니저, String 서버URI, String 이슈_키_또는_아이디, List<지라이슈_데이터> 하위이슈_목록) {

        Params params = new Params()
            .add("parent_id", 이슈_키_또는_아이디)
            .add("status_id", "*");

        List<Issue> 조회된_하위이슈_목록 = null;
        try {
            조회된_하위이슈_목록 = 레드마인_매니저.getIssueManager().getIssues(params).getResults();
        }
        catch (RedmineException e) {
            ErrorLogUtil.exceptionLoggingAndReturn(e, this.getClass().getName(),
                "레드마인_온프레미스 ["+ 서버URI +"] :: 이슈_키_또는_아이디 :: "
                    + 이슈_키_또는_아이디+ " :: 서브테스크_가져오기 오류");
            return;
        }

        if (조회된_하위이슈_목록 == null || 조회된_하위이슈_목록.isEmpty()) {
            log.info("레드마인_온프레미스 ["+ 서버URI +"] :: 이슈_키_또는_아이디 :: "
                + 이슈_키_또는_아이디 + " :: 서브테스크 이슈 데이터가 없습니다.");
            return;
        }

        for (Issue 이슈 : 조회된_하위이슈_목록) {
            지라이슈_데이터 지라이슈_데이터 = convertRedmineIssueToAlmIssue(레드마인_매니저, 이슈, 이슈_키_또는_아이디, 서버URI);
            if (지라이슈_데이터 != null) {
                하위이슈_목록.add(지라이슈_데이터);
                서브테스크_가져오기_재귀(레드마인_매니저, 서버URI, String.valueOf(이슈.getId()), 하위이슈_목록);
            }
        }
    }



    @Override
    public List<지라이슈_데이터> 증분서브테스크_가져오기(서버정보_데이터 서버정보, String 이슈_키_또는_아이디, String startDate, String endDate) {

        RedmineManager 레드마인_매니저 = 레드마인유틸.레드마인_온프레미스_통신기_생성(서버정보.getUri(), 서버정보.getPasswordOrToken());
        List<지라이슈_데이터> 하위이슈_목록 = new ArrayList<>();
        서브테스크_가져오기_재귀(레드마인_매니저, 서버정보.getUri(), 이슈_키_또는_아이디, 하위이슈_목록);


        // 업데이트된 이슈들만 필터링
        List<지라이슈_데이터> 업데이트된_하위이슈_목록 = 하위이슈_목록.stream()
            .filter(이슈 -> {

                String 업데이트 = 이슈.getFields().getUpdated();

                try {

                    ZonedDateTime 업데이트_날짜 = ZonedDateTime.parse(업데이트, DateTimeFormatter.ISO_OFFSET_DATE_TIME);

                    LocalDate updateLocalDate = 업데이트_날짜.toLocalDate();

                    return isDateInRange(updateLocalDate,toLocalDate(startDate),toLocalDate(endDate));

                } catch (Exception e) {
                    return false; // 파싱에 실패한 경우 필터링에서 제외
                }
            })
            .collect(Collectors.toList());

        return 업데이트된_하위이슈_목록;
    }

    private 지라이슈_데이터 convertRedmineIssueToAlmIssue(RedmineManager 레드마인_매니저, Issue 이슈, String 서버정보경로) {
        return convertRedmineIssueToAlmIssue(레드마인_매니저, 이슈, null, 서버정보경로);
    }

    private 지라이슈_데이터 convertRedmineIssueToAlmIssue(RedmineManager 레드마인_매니저, Issue 레드마인_이슈, String 부모_이슈키_또는_아이디, String 서버정보경로) {

        String 기본경로 = 레드마인유틸.서버정보경로_체크(서버정보경로);

        지라이슈_데이터 지라이슈_데이터 = new 지라이슈_데이터();

        Optional.ofNullable(레드마인_이슈.getId())
            .map(아이디 -> String.valueOf(아이디))
            .ifPresent(아이디 -> {
                지라이슈_데이터.setId(아이디);
                지라이슈_데이터.setKey(아이디);
                지라이슈_데이터.setSelf(기본경로 + 레드마인API_정보.아이디_대체하기(레드마인API_정보.getEndpoint().getIssue(), 아이디));
            });

        Optional.ofNullable(부모_이슈키_또는_아이디)
            .ifPresent(아이디 -> 지라이슈_데이터.setUpperKey(아이디));

        Optional<String> rawData = 레드마인유틸.toJson(레드마인_이슈);
        rawData.ifPresent(지라이슈_데이터::setRawData);

        지라이슈필드_데이터 설정한_지라이슈필드_데이터 = 레드마인이슈_지라이슈필드_데이터_매핑(레드마인_매니저, 레드마인_이슈, 기본경로);

        지라이슈_데이터.setFields(설정한_지라이슈필드_데이터);

        return 지라이슈_데이터;
    }

    private 지라이슈필드_데이터 레드마인이슈_지라이슈필드_데이터_매핑(RedmineManager 레드마인_매니저, Issue 이슈, String 기본경로) {
        지라이슈필드_데이터 지라이슈필드_데이터 = new 지라이슈필드_데이터();

        Optional.ofNullable(이슈.getProjectId())
            .map(아이디 -> String.valueOf(아이디))
            .ifPresent(아이디 -> {
                String 프로젝트_경로 = 기본경로 + 레드마인API_정보.아이디_대체하기(레드마인API_정보.getEndpoint().getProject(), 아이디);
                지라이슈필드_데이터.setProject(new ProjectDTO(프로젝트_경로, 아이디, 아이디, 이슈.getProjectName()));
            });

        Optional.ofNullable(이슈.getTracker().getId())
            .map(아이디 -> String.valueOf(아이디))
            .ifPresent(아이디 -> {
                String 이슈유형_경로 = 기본경로 + 레드마인API_정보.아이디_대체하기(레드마인API_정보.getEndpoint().getIssuetype(), 아이디);
                지라이슈필드_데이터.setIssuetype(new IssueTypeDTO(이슈유형_경로, 아이디, 이슈.getTracker().getName()));
            });

        Optional.ofNullable(이슈.getPriorityId())
            .map(아이디 -> String.valueOf(아이디))
            .ifPresent(아이디 -> {
                String 우선순위_경로 = 기본경로 + 레드마인API_정보.아이디_대체하기(레드마인API_정보.getEndpoint().getPriority(), 아이디);
                지라이슈필드_데이터.setPriority(new IssuePriorityDTO(우선순위_경로, 아이디, 이슈.getPriorityText()));
            });

        Optional.ofNullable(이슈.getStatusId())
            .map(아이디 -> String.valueOf(아이디))
            .ifPresent(아이디 -> {
                String 이슈상태_경로 = 기본경로 + 레드마인API_정보.아이디_대체하기(레드마인API_정보.getEndpoint().getIssuestatus(), 아이디);
                지라이슈필드_데이터.setStatus(new IssueStatusDTO(이슈상태_경로, 아이디, 이슈.getStatusName()));
            });

        Optional.ofNullable(이슈.getAuthorId())
            .map(아이디 -> String.valueOf(아이디))
            .ifPresent(아이디 -> {
                지라사용자_데이터 사용자_데이터 = new 지라사용자_데이터(아이디, 이슈.getAuthorName());
                지라이슈필드_데이터.setCreator(사용자_데이터);
                지라이슈필드_데이터.setReporter(사용자_데이터);
            });

        Optional.ofNullable(이슈.getAssigneeId())
            .map(아이디 -> String.valueOf(아이디))
            .ifPresent(아이디 -> {
                User 사용자정보 = 레드마인유틸.사용자정보_조회(레드마인_매니저, 아이디);

                if (사용자정보 != null && !사용자정보.getMail().isEmpty()) {
                    지라이슈필드_데이터.setAssignee(
                        new 지라사용자_데이터(아이디, 사용자정보.getMail(), 이슈.getAssigneeName()));
                }
                else {
                    지라이슈필드_데이터.setAssignee(new 지라사용자_데이터(아이디, 이슈.getAssigneeName()));
                }
            });

        Optional.ofNullable(이슈.getCreatedOn())
            .map(생성일 -> 날짜변환(생성일))
            .ifPresent(지라이슈필드_데이터::setCreated);

        Optional.ofNullable(이슈.getUpdatedOn())
            .map(업데이트일 -> 날짜변환(업데이트일))
            .ifPresent(지라이슈필드_데이터::setUpdated);

        Optional.ofNullable(이슈.getClosedOn())
            .map(해결책 -> 날짜변환(해결책))
            .ifPresent(지라이슈필드_데이터::setResolutiondate);

        지라이슈필드_데이터.setSummary(이슈.getSubject());

        return 지라이슈필드_데이터;
    }

    private String 날짜변환(Date 원본_날짜) {

        // 원본 형식
        DateTimeFormatter originalFormatter = DateTimeFormatter.ofPattern("EEE MMM dd HH:mm:ss zzz yyyy", Locale.US);

        // ZonedDateTime 객체로 파싱
        ZonedDateTime 날짜시간 = ZonedDateTime.parse(String.valueOf(원본_날짜), originalFormatter);

        // ISO 8601 형식으로 변환
        String 변환된_날짜시간 = 날짜시간.format(DateTimeFormatter.ISO_OFFSET_DATE_TIME);

        return 변환된_날짜시간;
    }

    private LocalDate 어제날짜_얻기() {
        return LocalDate.now().minusDays(1);
    }

}
